@@ -1,7 +1,8 @@
-from typing import Tuple
+from typing import Tuple, Any
 
 import imblearn # type: ignore
-import numpy as np # type: ignore
+import numpy as np
+import numpy.typing as npt
 from .base import BaseBalancer
 
 class RandomOverSampler(BaseBalancer):
@@ -13,6 +14,6 @@ class RandomOverSampler(BaseBalancer):
         super().__init__()
         self.ros = imblearn.over_sampling.RandomOverSampler(random_state=random_state)
 
-    def resample(self, x_data: np.ndarray, y_data: np.ndarray) -> Tuple[np.ndarray, np.ndarray]:
+    def resample(self, x_data: npt.NDArray[Any], y_data: npt.NDArray[Any]) -> Tuple[npt.NDArray[Any], npt.NDArray[Any]]:
         x_resampled, y_resampled = self.ros.fit_resample(x_data, y_data) # type: ignore
         return x_resampled, y_resampled # type: ignore
\ No newline at end of file
