@@ -1,42 +1,32 @@
-from dataclasses import dataclass
+import logging
 from pathlib import Path
-from typing import (
-    Any,
-    Mapping,
-    Tuple,
-    TypeVar,
-    Union,
-)
+from typing import Any, Callable, Mapping, TypeVar, Union
 from uuid import UUID
 
 import numpy as np
+import numpy.typing as npt
 import pandas as pd
 from instancelib import TextInstance
 from instancelib.ingest.spreadsheet import read_csv_dataset
+from instancelib.typehints.typevars import KT, LT
+from ..typehints.typevars import IT
 
-from allib.analysis.experiments import ExperimentIterator
-from allib.analysis.tarplotter import ModelStatsTar, TarExperimentPlotter
-from allib.configurations.base import STOP_REPOSITORY
-from allib.stopcriterion.catalog import StopCriterionCatalog
-
-from ..analysis.analysis import process_performance
-from ..analysis.initialization import SeparateInitializer
-from ..analysis.plotter import AbstractPlotter, BinaryPlotter
-from ..analysis.simulation import TarSimulator, initialize, simulate
-from ..stopcriterion.base import AbstractStopCriterion
+from ..analysis.experiments import ExperimentIterator
+from ..analysis.initialization import Initializer, SeparateInitializer
+from ..analysis.simulation import TarSimulator, initialize_tar_simulation
+from ..analysis.tarplotter import ModelStatsTar, TarExperimentPlotter
 from ..environment import AbstractEnvironment
 from ..environment.memory import MemoryEnvironment
 from ..estimation.base import AbstractEstimator
-from ..estimation.rasch import ParametricRasch
-from ..estimation.rasch_python import EMRaschRidgePython
 from ..module.factory import MainFactory
+from ..stopcriterion.base import AbstractStopCriterion
 from ..utils.func import list_unzip3
-import logging
 
 POS = "Relevant"
 NEG = "Irrelevant"
 LOGGER = logging.getLogger(__name__)
 
+
 def binary_mapper(value: Any) -> str:
     return POS if value == 1 else NEG
 
@@ -48,10 +38,10 @@ LT = TypeVar("LT")
 def read_review_dataset(
     path: Path,
 ) -> AbstractEnvironment[
-    TextInstance[Union[int, UUID], np.ndarray],
+    TextInstance[Union[int, UUID], npt.NDArray[Any]],
     Union[int, UUID],
     str,
-    np.ndarray,
+    npt.NDArray[Any],
     str,
     str,
 ]:
@@ -64,7 +54,7 @@ def read_review_dataset(
 
     Returns
     -------
-    MemoryEnvironment[int, str, np.ndarray, str]
+    MemoryEnvironment[int, str, npt.NDArray[Any], str]
         A MemoryEnvironment. The labels that
     """
     df = pd.read_csv(path)
@@ -86,15 +76,14 @@ def read_review_dataset(
     return al_env
 
 
-
-
 def benchmark(
-    path: Path,
+    env: AbstractEnvironment[IT, KT, Any, Any, Any, str],
     output_path: Path,
     output_pdf_path: Path,
     al_config: Mapping[str, Any],
     fe_config: Mapping[str, Any],
-    estimators: Mapping[str, AbstractEstimator[Any, Any, Any, Any, Any, str]],
+    initializer_builder: Callable[..., Initializer[IT, KT, str]],
+    estimators: Mapping[str, AbstractEstimator[IT, KT, Any, Any, Any, str]],
     stopcriteria: Mapping[str, AbstractStopCriterion[str]],
     pos_label: str,
     neg_label: str,
@@ -102,10 +91,11 @@ def benchmark(
     stop_interval: Union[int, Mapping[str, int]] = 10,
     estimation_interval: Union[int, Mapping[str, int]] = 10,
 ) -> TarExperimentPlotter[str]:
-    env = read_review_dataset(path)
     factory = MainFactory()
-    initializer = SeparateInitializer(env, 1)
-    al, _ = initialize(factory, al_config, fe_config, initializer, env)
+    initializer = initializer_builder(pos_label=pos_label, neg_label=neg_label)
+    al, _ = initialize_tar_simulation(
+        factory, al_config, fe_config, initializer, env, pos_label, neg_label
+    )
     exp = ExperimentIterator(
         al,
         pos_label,
@@ -117,11 +107,12 @@ def benchmark(
         estimation_interval,
     )
     plotter = ModelStatsTar(POS, NEG)
-    simulator = TarSimulator(exp, plotter, output_path=output_path, output_pdf_path=output_pdf_path)
+    simulator = TarSimulator(
+        exp, plotter, output_path=output_path, output_pdf_path=output_pdf_path
+    )
     try:
         simulator.simulate()
     except Exception as e:
         LOGGER.error("Exited with %s", e)
         pass
     return plotter
-
